Package: rts2
Title: Log-Gaussian Cox Process Models with Approximations
Version: 1.0.3
Date: 2026-06-06
Authors@R: 
    person(given = "Sam",
           family = "Watson",
           role = c("aut", "cre"),
           email = "s.i.watson@bham.ac.uk",
           comment = c(ORCID = "0000-0002-8972-769X"))
Description: Supports modelling case data to facilitate. The package provides automated computational grid generation over
    an area of interest with methods to map covariates between geographies, model fitting including spatially aggregated case counts, 
    and predictions and visualisation. Monte Carlo maximum likelihood is the main fitting method with a low-rank approximation for Gaussian processes 
    described by Solin and Särkkä (2020) <doi:10.1007/s11222-019-09886-w> and a stochastic partial differential equation approximation. Bayesian methods 
    are also provided for some methods. Log-Gaussian Cox Processes are described by 
    Diggle et al. (2013) <doi:10.1214/13-STS441>. 
License: CC BY-SA 4.0
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Biarch: true
Depends: R (>= 3.5.0), sf (>= 1.0-14)
Imports: methods, R6, Rcpp (>= 0.12.0), RcppParallel (>= 5.0.1), rstan
        (>= 2.30.0), rstantools (>= 2.1.1), lubridate (>= 1.9.0), stars
        (>= 0.6-1), raster (>= 3.6-1), glmmrBase (>= 1.3.0), spdep,
        fmesher, FNN, quadprog
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.30.0), StanHeaders (>=
        2.32.0), glmmrBase (>= 1.3.0)
SystemRequirements: GNU make
NeedsCompilation: yes
Packaged: 2026-06-06 16:34:36 UTC; samue
Author: Sam Watson [aut, cre] (ORCID: <https://orcid.org/0000-0002-8972-769X>)
Maintainer: Sam Watson <s.i.watson@bham.ac.uk>
Repository: CRAN
Date/Publication: 2026-06-07 05:40:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-06-07 06:34:58 UTC; unix
Archs: rts2.so.dSYM
