S3method(print,rtsFit)
S3method(print,rtsFitSummary)
S3method(summary,rtsFit)
S3method(coef,rtsFit)
S3method(logLik,rtsFit)
S3method(family,rtsFit)
S3method(formula,rtsFit)
S3method(vcov,rtsFit)
S3method(fitted,rtsFit)
S3method(residuals,rtsFit)
S3method(predict,rtsFit)
S3method(summary,grid)
S3method(family,grid)
S3method(predict,grid)
S3method(residuals,grid)
S3method(vcov,grid)
S3method(formula,grid)
export(covariance.parameters)
export(create_points)
export(fixed.effects)
export(grid)
export(progress_bar)
export(random.effects)
import(Rcpp)
import(lubridate)
import(methods)
import(sf)
importFrom(R6,R6Class)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,model.matrix)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(utils,capture.output)
importFrom(utils,stack)
importFrom("stats", "as.formula", "formula", "residuals", "vcov")
importFrom(stats,family)
importFrom("utils", "head")
importFrom(stars,read_stars)
importFrom(raster, raster)
importFrom(raster, writeRaster)
importFrom(stats, optim)
importFrom(stats, var)
importFrom(quadprog, solve.QP)
importFrom(spdep, nb2mat)
importFrom(glmmrBase, Model)
importFrom(fmesher, fm_int)
importFrom(fmesher, fm_crs)
importFrom(fmesher, fm_mesh_2d)
importFrom(fmesher, fm_basis)
importFrom(fmesher, fm_fem)
importFrom(FNN, get.knnx)
useDynLib(rts2, .registration = TRUE)
