# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

regionModel__new <- function(formula_, data_, colnames_, X_, y_, niter_, type = 0L) {
    .Call(`_rts2_regionModel__new`, formula_, data_, colnames_, X_, y_, niter_, type)
}

regionModel_ar__new <- function(formula_, data_, colnames_, X_, y_, niter_, T_) {
    .Call(`_rts2_regionModel_ar__new`, formula_, data_, colnames_, X_, y_, niter_, T_)
}

regionModel_hsgp__new <- function(formula_, data_, colnames_, X_, y_, niter_, m_, L_boundary_) {
    .Call(`_rts2_regionModel_hsgp__new`, formula_, data_, colnames_, X_, y_, niter_, m_, L_boundary_)
}

regionModel_spde__new <- function(formula_, data_, colnames_, X_, y_, niter_, A_loc_, C_diag_, G_, alpha_ = 2L) {
    .Call(`_rts2_regionModel_spde__new`, formula_, data_, colnames_, X_, y_, niter_, A_loc_, C_diag_, G_, alpha_)
}

regionModel__zu_variance_full <- function(xp, type) {
    .Call(`_rts2_regionModel__zu_variance_full`, xp, type)
}

regionModel_spde__zu_variance_full <- function(xp, A_pred_, X_pred_) {
    .Call(`_rts2_regionModel_spde__zu_variance_full`, xp, A_pred_, X_pred_)
}

regionModel__set_weights <- function(xp, i, p, x, nrow, ncol, type = 0L) {
    invisible(.Call(`_rts2_regionModel__set_weights`, xp, i, p, x, nrow, ncol, type))
}

regionModel__set_theta <- function(xp, theta, type = 0L) {
    invisible(.Call(`_rts2_regionModel__set_theta`, xp, theta, type))
}

regionModel__set_offset <- function(xp, offset, type = 0L) {
    invisible(.Call(`_rts2_regionModel__set_offset`, xp, offset, type))
}

regionModel__fit <- function(xp, tol, max_iter, hist, k0, type = 0L) {
    invisible(.Call(`_rts2_regionModel__fit`, xp, tol, max_iter, hist, k0, type))
}

regionModel__information_matrix <- function(xp, mc = TRUE, type = 0L) {
    .Call(`_rts2_regionModel__information_matrix`, xp, mc, type)
}

regionModel__information_matrix_theta <- function(xp, type = 0L) {
    .Call(`_rts2_regionModel__information_matrix_theta`, xp, type)
}

regionModel__u <- function(xp, scaled = TRUE, type = 0L) {
    .Call(`_rts2_regionModel__u`, xp, scaled, type)
}

regionModel__get_beta <- function(xp, type = 0L) {
    .Call(`_rts2_regionModel__get_beta`, xp, type)
}

regionModel__get_weights <- function(xp, type = 0L) {
    .Call(`_rts2_regionModel__get_weights`, xp, type)
}

regionModel__log_likelihood <- function(xp, type = 0L) {
    .Call(`_rts2_regionModel__log_likelihood`, xp, type)
}

regionModel__get_theta <- function(xp, type = 0L) {
    .Call(`_rts2_regionModel__get_theta`, xp, type)
}

regionModel__set_trace <- function(xp, trace, type = 0L) {
    invisible(.Call(`_rts2_regionModel__set_trace`, xp, trace, type))
}

max_dist <- function(x) {
    .Call(`_rts2_max_dist`, x)
}

semivariogram <- function(x, offs, y, nbins, nT) {
    .Call(`_rts2_semivariogram`, x, offs, y, nbins, nT)
}

GridData__NN <- function(ptr_) {
    .Call(`_rts2_GridData__NN`, ptr_)
}

GridData__gen_NN <- function(ptr_, m_) {
    invisible(.Call(`_rts2_GridData__gen_NN`, ptr_, m_))
}

GridData__new <- function(x_, t_) {
    .Call(`_rts2_GridData__new`, x_, t_)
}

